/*! 
\file 
\brief Audio Stream

Interface for working with audio streams
\date 21 November 2003
\author Evgeny Karpov
*/

/*
    $Id: audiostream.h,v 1.4 2003/11/26 15:50:29 ekarpov Exp $
*/

#ifndef __AUDIOSTREAM_H__
#define __AUDIOSTREAM_H__

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Structures =============================*/

/*! audio parameters */
typedef struct {

    int codec ;          /*!< codec type used for data reading */
    int channels ;       /*!< number of channels (mono or stereo)    */
    int samples_per_sec ;/*!< number of samples per second (in Hertz)*/
    int bits_per_sample ;/*!< bits per sample                        */
    int sign ;           /*!< signed or unsigned samples */

} AudioParams;

/*! Stream modes */
typedef enum {

    ASTREAM_MODE_READ, /*!< read audio data */
    ASTREAM_MODE_WRITE /*!< write audio data */

} AStreamMode ;

/*! Stream type */
typedef enum {

    ASTREAM_TYPE_FILE, /*!< file stream */
    ASTREAM_TYPE_DEV   /*!< device stream */

} AStreamType ;

/*======================== Structures =============================*/

typedef void * HASTREAM ;

/*-----------------------------------------------------------------*/

/* audio stream flags */
#define ASTREAM_ASYNC       0x0001      /* all operations are asynchronous */
#define ASTREAM_KEEP_DATA   0x0002      /* all data will be kept */
#define ASTREAM_EOS         0x0004      /* stream position is at the end */

/*======================== Functions ==============================*/

/* open stream */
HASTREAM asOpen(AStreamType type, AStreamMode mode,
                 AudioParams * audio_params, char * file_name) ;
/* close all streams */
void asClose(HASTREAM as) ;

/* reading from stream */
int asRead(HASTREAM as,void * buf,int samples) ;

/* writing to stream */
int asWrite(HASTREAM as,void * buf,int samples) ;

/* manipulating with stream */
int asEndOfStream(HASTREAM as) ;

/* return the size of data in stream in samples (if possible) */
int asGetSize(HASTREAM as) ; 

/* stream parameters */
AStreamType asGetType(HASTREAM as) ;
AStreamMode asGetMode(HASTREAM as) ;

/* signal parameters */
int asGetChannels(HASTREAM as) ;
int asGetRate(HASTREAM as) ;
int asGetBitsPerSample(HASTREAM as) ;

/* specific reading functions */
int asReadDouble(HASTREAM as,double * buff,int samples) ;

/* specific functions for asynchronous streams */
int asStart(HASTREAM as) ;
int asPause(HASTREAM as) ;
int asResume(HASTREAM as) ;
int asStop(HASTREAM as) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __AUDIOSTREAM_H__ */
