/*!
\file dsp.h
\brief Digital signal processing interface

Interface for working with several
well-known digital signal processing algorithms\n
$Id: dsp.h,v 1.2 2003/11/11 20:38:45 juhani Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef DSP_H
#define DSP_H

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Functions ==============================*/

/*
Inner product
 */
double dspInnerProduct(
	double * factor1,
	double * factor2,
	int size);

/*
Pointwise multiplication
 */
int dspMultiply(
        double * factor1,
        double * factor2,
        double * output,
        int size);

/*
Convolution
 */
int dspConvolution(
	double * input,
	int input_size,
	double * kernel,
	int kernel_size,
	double * output) ;

/*
Discrete Cosine Transformation
 */
int dspDCT(
	double * input,
	int input_size,
	double * output,
	int output_size) ;


/*
DC Removal
 */
int dspDCRemove(
	double * input,
	double * output,
	int size,
	void * params) ;


/*
First Order Difference Filtering
 */
int dspHighEmphFilter(
	double * input,
	double * output,
	int size,
	void * params) ;


/*
Number of Zero Crossings
 */
int dspZCR(
	double * input,
	int size) ;


/*
Vector Energy
 */
double dspEnergy(
	double * input,
	int size) ;


/*
Power of a Finite Digital Signal
 */
double dspPower(
	double * input,
	int size) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* DSP_H */
