/*! 
\file fftdct.h
\brief A generic FFT and DCT library interface,
used by MFCC module.

This is an interface for FFT and DCT.\n
$Id: fftdct.h,v 1.1 2003/12/18 10:29:46 juhani Exp $
\date 2003/12
\author Juhani Saastamoinen
*/
#ifndef FFTDCT_H
#define FFTDCT_H


#include <fftw.h>


typedef struct
{
	int input_size; /*!< FFT size for FFTW */
    fftw_plan      fftw_plan ; /*!< FFTW FFT plan handle */
    fftw_complex * fftw_out ;  /*!< FFTW FFT output */
    fftw_complex * fftw_in ;   /*!< FFTW FFT input */
} FFTProcessor;


typedef struct
{
	int input_size;
	int spectrum_size;
	double** cos_functions;
} DCTProcessor;


FFTProcessor* fftCreate(int input_size);
void fftDestroy(FFTProcessor* fft);
int fftGetInputSize(FFTProcessor* fft);
int fftCompute(FFTProcessor* fft, double* input, double* spectrum);


DCTProcessor* dctCreate(int input_size, int spectrum_size);
void dctDestroy(DCTProcessor* fft);
int dctGetInputSize(DCTProcessor* fft);
int dctGetSpectrumSize(DCTProcessor* fft);
int dctCompute(DCTProcessor* fft, double* input, double* spectrum);


#endif /* #ifndef FFTDCT_H */

