/*!
\file sremoval.h
\brief Silence removal interface

Interface for working with several
algorithms for silence removal.\n
$Id: sremoval.h,v 1.1 2003/10/31 15:36:53 juhani Exp $
\author Evgeny Karpov
*/

#ifndef SREMOVAL_H
#define SREMOVAL_H

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Structures =============================*/

/*!
\brief Silence Removal Algorithm Types

Classified silence removal algorithms.
*/
typedef enum {

    SRTYPE_BPBP, /*!< Burileanu, Pascalin, Burileanu and Puchiu */
    SRTYPE_KINN  /*!< Kinnunen algorithm */

} SRTypes ;

/*!
\brief BPBP Algorithm Params

Parameter type for silence removal algorithms of the type SRTYPE_BPBP.
\sa SRTypes
*/
typedef struct {

    double thr1 ; /*!< first energy threshold */
    double thr2 ; /*!< second energy threshold */

} SRParams_BPBP ;

/*======================== Structures =============================*/

/*!
\brief Silence Remover Handle

A handle is defined as a void pointer.
In reality it is a pointer to a silence remover
struct/object.
*/
typedef void * HSREM ;

/*======================== Functions ==============================*/

/*
Create a New Silence Remover
*/
HSREM srCreate(
	SRTypes type,
	void * params) ;

/*
Destroy a Silence Remover
*/
void srDestroy(
	HSREM sr) ;

/*
Silence Remover Training
 */
int srTrain(
	HSREM sr,
	double * frame,
	int size) ;

/*
Silence Checking
*/
int srIsSilent(
	HSREM sr,
	double * frame,
	int size) ;

/*
Start the next silence removal with the initial state
*/
int srSetInitial(
	HSREM sr) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* SREMOVAL_H */
