/*!
\file winfunc.h
\brief Signal windowing functions interface

Interface for computing several
standard signal windowing functions\n
$Id: winfunc.h,v 1.1 2003/10/31 15:36:53 juhani Exp $
\author Evgeny Karpov
*/

#ifndef WINFUNC_H
#define WINFUNC_H

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------*/


/*!
\brief Window function pointer type

A window function always takes two parameters:
pointer to output data and window vector length.
Different window functions evaluate different vectors.
*/
typedef int (*WINFUNC) (double * coeff,int win_size) ;


/*======================== Functions ==============================*/


/*
Hamming window
 */
int wfHamming(
	double * coeff,
	int win_size) ;


/*
Bartlett window
 */
int wfBarlett(
	double * coeff,
	int win_size) ;


/*
Hanning window
 */
int wfHanning(
	double * coeff,
	int win_size) ;


/*
Blackman window
 */
int wfBlackman(double * coeff,int win_size) ;


/*
Gaussian window
 */
int wfGaussian(double * coeff,int win_size) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* WINFUNC_H */
