/*! 
\file fixmath.h
\brief Interface to math library functions in fixed point arithmetic.

This modules has implementations for some functions most commonly
seen in any standard C-library. The functions are implemented
using fixed point arithmetic only scaling the variables
appropriately to minimize relative error.\n
$Id: fixmath.h,v 1.18 2003/12/23 09:49:01 juhani Exp $
\date 2003/11
\author Juhani Saastamoinen
*/
#ifndef FIXMATH_H
#define FIXMATH_H


#if defined(__cplusplus)
extern "C" {
#endif


#ifdef TInt16
#	define INT8 TInt8
#	define INT16 TInt16
#	define INT32 TInt32
#	define UINT8 TUInt8
#	define UINT16 TUInt16
#	define UINT32 TUInt32
#	define FLOAT TReal
#else
#	define INT8 signed char
#	define INT16 signed short int
#	define INT32 signed long int
#	define UINT8 unsigned char 
#	define UINT16 unsigned short int
#	define UINT32 unsigned long int
#	define FLOAT double
#endif


double frac_dbl(INT32 x, UINT32 f);
const char* frac_str(INT32 x, UINT32 f);
	

INT16 frac_sum(
	INT32 x1, UINT32 f1,
	INT32 x2, UINT32 f2,
	INT32* y, UINT32* f);
INT16 frac_mul(
	INT32 x1, UINT32 f1,
	INT32 x2, UINT32 f2,
	INT32* y, UINT32* f);
INT16 frac_div(
	INT32 x1, UINT32 f1,
	INT32 x2, UINT32 f2,
	INT32* y, UINT32* f);


INT32 frac_pow2(INT16 x, UINT32 y, UINT32* frac);
INT16 frac_sqrt(UINT32 x, UINT32* frac, UINT16 nbits);


#define INT_NBITS 32U
#define INT_ONE ((UINT32) (1U << (INT_NBITS - 1)))
#define FRAC_NBITS 16U
#define FRAC_ONE ((UINT32) (1U << (FRAC_NBITS - 1)))

INT16 frac_log2_init(UINT16 nbits);
INT8 frac_log2(UINT32 x, UINT32* frac, UINT16 nbits);


#if defined(__cplusplus)
}
#endif


#endif /* #ifndef FIXMATH_H */

