#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>


#include "fixmath.h"


int test_sum(INT32 minindex, INT32 maxindex,
		     UINT32 minfrac, UINT32 maxfrac,
			 INT32 iop2, UINT32 fop2)
{
	INT32 x, maxx;
	INT32 f;
	INT32 f32;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;
	UINT32 fracin, maxfracin;

	printf("test_sum:\n");

	maxx = minindex;
	maxfracin = 0;
	maxer = -1.0;
	f = 1;
	f32 = 1;
    e = 1.0;
	real_f = 1.0;
    er = 1.0;

	for (x = minindex; x < maxindex; x++)	
	{
		for (fracin = minfrac; fracin < maxfrac; fracin++)
		{
			real_f = frac_dbl(x, fracin) + frac_dbl(iop2, fop2);
			f = frac_sum(x, fracin, iop2, fop2, &f32, &frac32);
			if (f < 0)
			{
				fprintf(stderr, "main: frac_sum(%ld, %lu, %ld, %lu) failed\n",
						x, fracin, iop2, fop2);
				return 1;
			}

			e = real_f - frac_dbl(f32, frac32);
			if (frac_dbl(f32, frac32) < DBL_EPSILON)
			{
				er = fabs(e);
			}
			else if (fabs(e) > DBL_EPSILON)
			{
				er = fabs(e) / fabs(real_f);
			}
			else 
			{
				er = 0.0;
			}

#if defined(DEBUG_PRINT)
			fprintf(stdout, "%.10f + %.10f = %.10f %.10f %7.3E\n",
				frac_dbl(x, fracin),
				frac_dbl(iop2, fop2),
				frac_dbl(f32, frac32),
				real_f, er);
#endif

			if (er > 1.0)
			{
				/* something is wrong if relative error > 1 */
				fprintf(stderr,
  	              "main: x = %ld, relative error = %16.10f\n",
					x, er);
				return 1;
			}
			if (er > maxer)
			{
				maxer = er;
				maxx = x;
				maxfracin = fracin;
			}

		}
	}

	printf(
      "Maximum relative error is %16.10E in x = %16.10f\n",
      maxer, frac_dbl(maxx, maxfracin));

	real_f = frac_dbl(maxx, maxfracin) + frac_dbl(maxx, maxfracin);
	printf(
      "  %16.10f + %16.10f = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  real_f);

	f = frac_sum(maxx, maxfracin, iop2, fop2, &f32, &frac32);
	printf(
      "  frac_sum(%16.10f, %16.10f) = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  frac_dbl(f32, frac32));

	printf("  (x = %ld, y = %lu gives maximum error)\n", maxx, maxfracin);
	
	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int test_mul(INT32 minindex, INT32 maxindex,
		     UINT32 minfrac, UINT32 maxfrac,
			 INT32 iop2, UINT32 fop2)
{
	INT32 x, maxx;
	INT32 f;
	INT32 f32;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;
	UINT32 fracin, maxfracin;

	printf("\ntest_mul:\n");

	maxx = minindex;
	maxfracin = 0;
	maxer = -1.0;
	f = 1;
	f32 = 1;
    e = 1.0;
	real_f = 1.0;
    er = 1.0;

	for (x = minindex; x <= maxindex; x++)	
	{
		for (fracin = minfrac; fracin < maxfrac; fracin++)
		{
			real_f = frac_dbl(x, fracin) * frac_dbl(iop2, fop2);
			f = frac_mul(x, fracin, iop2, fop2, &f32, &frac32);
			if (f < 0)
			{
				fprintf(stderr, "main: frac_mul(%ld, %lu, %ld, %lu) failed\n",
						x, fracin, iop2, fop2);
				return 1;
			}

			e = real_f - frac_dbl(f32, frac32);
			if (frac_dbl(f32, frac32) < DBL_EPSILON)
			{
				er = fabs(e);
			}
			else if (fabs(e) > DBL_EPSILON)
			{
				er = fabs(e) / fabs(real_f);
			}
			else 
			{
				er = 0.0;
			}

#if defined(DEBUG_PRINT)
			fprintf(stdout, "%16.10f * %16.10f = %16.10f %16.10f %16.10f\n",
				frac_dbl(x, fracin),
				frac_dbl(iop2, fop2),
				frac_dbl(f32, frac32),
				real_f, er);
#endif

			if (er > 1.0)
			{
				/* something is wrong if relative error > 1 */
				fprintf(stderr,
  	              "main: x = %ld, relative error = %16.10f\n",
					x, er);
				return 1;
			}
			if (er > maxer)
			{
				maxer = er;
				maxx = x;
				maxfracin = fracin;
			}

		}
	}

	printf(
      "Maximum relative error is %16.10E in x = %16.10f\n",
      maxer, frac_dbl(maxx, maxfracin));

	real_f = frac_dbl(maxx, maxfracin) * frac_dbl(iop2, fop2);
	printf(
      "  %16.10f * %16.10f = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  real_f);

	f = frac_mul(maxx, maxfracin, iop2, fop2, &f32, &frac32);
	printf(
      "  frac_mul(%16.10f, %16.10f) = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  frac_dbl(f32, frac32));

	printf("  (x = %ld, y = %lu gives maximum error)\n", maxx, maxfracin);
	
	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int test_div(
	INT32 minindex, INT32 maxindex,
	UINT32 minfrac, UINT32 maxfrac,
	INT32 iop2, UINT32 fop2)
{
	INT32 x, maxx;
	INT32 f;
	INT32 f32;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;
	UINT32 fracin, maxfracin;

	printf("\ntest_div:\n");

	maxx = minindex;
	maxfracin = 0;
	maxer = -1.0;
	f = 1;
	f32 = 1;
    e = 1.0;
	real_f = 1.0;
    er = 1.0;

	for (x = minindex; x <= maxindex; x++)	
	{
		for (fracin = minfrac; fracin <= maxfrac; fracin++)
		{
			real_f = frac_dbl(x, fracin) / frac_dbl(iop2, fop2);
			f = frac_div(x, fracin, iop2, fop2, &f32, &frac32);
			if (f < 0)
			{
				fprintf(stderr, "main: frac_div(%ld, %lu, %ld, %lu) failed\n",
						x, fracin, iop2, fop2);
				return 1;
			}

			e = real_f - frac_dbl(f32, frac32);
			if (frac_dbl(f32, frac32) < DBL_EPSILON)
			{
				er = fabs(e);
			}
			else if (fabs(e) > DBL_EPSILON)
			{
				er = fabs(e) / fabs(real_f);
			}
			else 
			{
				er = 0.0;
			}

#if defined(DEBUG_PRINT)
			fprintf(stdout, "%16.10f / %ld = %16.10f %16.10f %16.10f\n",
				frac_dbl(x, fracin), x,
				frac_dbl(f32, frac32),
				real_f, er);
#endif

			if (er > 1.0)
			{
				/* something is wrong if relative error > 1 */
				fprintf(stderr,
  	              "main: x = %ld, relative error = %16.10f\n",
					x, er);
				return 1;
			}
			if (er > maxer)
			{
				maxer = er;
				maxx = x;
				maxfracin = fracin;
			}

		}
	}

	printf(
      "Maximum relative error is %16.10E in x = %16.10f\n",
      maxer, frac_dbl(maxx, maxfracin));

	f = frac_div(maxx, maxfracin, iop2, fop2, &f32, &frac32);
	if (f < 0)
	{
		fprintf(stderr, "main: frac_div(%ld, %lu, %ld, %lu) failed\n",
				maxx, maxfracin, iop2, fop2);
		return 1;
	}

	printf(
      "  frac_div(%16.10f, %16.10f) = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  frac_dbl(f32, frac32));

	real_f = frac_dbl(maxx, maxfracin) / frac_dbl(iop2, fop2);
	printf(
      "  %16.10f / %16.10f = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(iop2, fop2),
	  real_f);

	printf("  (x = %ld, y = %lu gives maximum error)\n", maxx, maxfracin);
	
	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int test_pow2(INT32 minindex, INT32 maxindex)
{
	INT32 x, maxx;
	INT32 f;
	INT32 f32;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;
	UINT32 fracin, maxfracin;

	printf("\ntest_pow2:\n");

	maxx = minindex;
	maxfracin = 0;
	maxer = -1.0;
	f = 1;
	f32 = 1;
    e = 1.0;
	real_f = 1.0;
    er = 1.0;

	for (x = minindex; (x > 0) && (x <= maxindex); x++)	
	{
		for (fracin = 0; fracin < FRAC_ONE; fracin++)
		{
			real_f = pow(2.0, frac_dbl(x, fracin));
			f32 = frac_pow2(x, fracin, &frac32);
			if (f32 < 0)
			{
				fprintf(stderr, "main: frac_pow2(%ld, %lu) failed\n",
				  x, fracin);
				return 1;
			}

			e = real_f - frac_dbl(f32, frac32);
			if (fabs(e) > DBL_EPSILON)
			{
				er = fabs(e) / fabs(real_f);
			}
			else 
			{
				er = 0.0;
			}

#if defined(DEBUG_PRINT)
			fprintf(stdout, "pow2(%16.10f) = %16.10f %16.10f %16.10f\n",
			  frac_dbl(x, fracin), frac_dbl(f32, frac32), real_f, er);
#endif

			if (er > 1.0)
			{
				/* something is wrong if relative error > 1 */
				fprintf(stderr,
  	              "main: x = %ld, relative error = %16.10f\n",
					x, er);
				return 1;
			}
			if (er > maxer)
			{
				maxer = er;
				maxx = x;
				maxfracin = fracin;
			}

		}
	}

	printf(
      "Maximum relative error is %16.10E in x = %16.10f\n",
      maxer, frac_dbl(maxx, maxfracin));

	f32 = frac_pow2(maxx, maxfracin, &frac32);
	printf(
      "  frac_pow2(%16.10f) = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  frac_dbl(f32, frac32));
	printf(
      "  pow2(%.12g) = %16.10f\n",
	  frac_dbl(maxx, maxfracin),
	  pow(2.0, frac_dbl(maxx, maxfracin)));

	printf("  (x = %ld, y = %lu gives maximum error)\n", maxx, maxfracin);
	
	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int test_sqrt(INT32 minindex, INT32 maxindex, INT16 nbits)
{
	INT32 x, maxx;
	INT32 f;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;

	printf("\ntest_sqrt(%ld, %ld, %d):\n", minindex, maxindex, nbits);

	maxx = minindex;
	maxer = -1.0;
	f = 1;
	e = 1.0;
	real_f = 1.0;
	er = 1.0;

	for (x = minindex; (x >= 0) && (x <= maxindex); x++)
	{
		real_f = sqrt(x);
		f = frac_sqrt(x, &frac32, nbits);
		if (f < 0)
		{
			fprintf(stderr,
			  "main: frac_sqrt(x) failed, x = %ld\n", x);
			return 1;
		}

		e = real_f - frac_dbl(f, frac32);
		er = fabs(e) / fabs(real_f);
		if (er > 1.0)
		{
			/* something is wrong if relative error > 1 */
			fprintf(stderr,
			    "main: x = %ld, relative error = %16.10f\n",
			    x, er);
			return 1;
		}

#if defined(DEBUG_PRINT)
		fprintf(stdout, "%10ld %16.10f %16.10f %16.10f\n",
		    x, frac_dbl(f, frac32),
		    real_f, er);
#endif

		if (er > maxer)
		{
			maxer = er;
			maxx = x;
		}
	}

	printf(
      "Maximum relative error is %16.10E in x = %ld\n",
      maxer, maxx);

	f = frac_sqrt(maxx, &frac32, nbits);
	printf(
      "  frac_sqrt(%ld) = %16.10f\n",
	  maxx,
	  frac_dbl(f, frac32));

	real_f = sqrt((double) maxx);
	printf(
      "  sqrt(%ld) = %16.10f\n",
	  maxx, real_f);

	printf("  (x = %ld gives maximum error)\n", maxx);

	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int test_log2(INT32 minindex, INT32 maxindex, INT16 nbits)
{
	INT32 x, maxx;
	INT32 f;
	UINT32 frac32;
	double real_f;
	double e, er, maxer;
	int exp;

	/* log2 */
	printf("\ntest_log2:\n");

	maxx = minindex;
	maxer = -1.0;
	f = 1;
    e = 1.0;
	real_f = 1.0;
    er = 1.0;

	for (x = minindex; (x >= minindex) && (x <= maxindex); x++)
	{
		real_f = log((double) x) / log(2.0);
		f = frac_log2(x, &frac32, nbits);
		if (f < 0)
		{
			fprintf(stderr,
         	    "main: frac_log2(x) failed, x = %ld\n", x);
			return 1;
		}

		e = real_f - frac_dbl(f, frac32);
		if (fabs(e) > DBL_EPSILON)
		{
			er = fabs(e) / fabs(real_f);
		}
		else 
		{
			er = 0.0;
		}

#if defined(DEBUG_PRINT)
		fprintf(stdout, "%10ld %16.10f %16.10f %16.10f\n",
				x, frac_dbl(f, frac32),
				real_f, er);
#endif

		if (er > 1.0)
		{
			/* something is wrong if relative error > 1 */
			fprintf(stderr,
                "main: x = %ld, relative error = %16.10f\n",
				x, er);
			return 1;
		}
		if (er > maxer)
		{
			maxer = er;
			maxx = x;
		}

		/*
		printf("%6d, %6d, %16.10f, %16.10f, %16.10f\n",
                           x, f, real_f, e, er);
	}
	printf("\n");
	*/
	}

	printf(
	    "Maximum relative error is %16.10E in x = %ld\n",
	    maxer, maxx);

	f = frac_log2(maxx, &frac32, nbits);
	printf(
      "  frac_log2(%ld) = %16.10f\n",
	  maxx,
	  frac_dbl(f, frac32));

	real_f = log((double) maxx) / log(2.0);
	printf(
      "  log2(%ld) = %16.10f\n",
	  maxx, real_f);

	printf("  (x = %ld gives maximum error)\n", maxx);
	
	maxer = frexp(maxer, &exp);
	printf("maxer = %g_10 x 2^(%d_10)\n", maxer, exp);

	return 0;
}


int main(int argc, char* argv[])
{
	int result;

	int nbits;
	INT32 minindex, maxindex;
	UINT32 minfrac, maxfrac;
	INT32 iop2;
	UINT32 fop2;

	nbits = 8;
	minindex = 1;
	maxindex = 10;
	minfrac = 0;
	maxfrac = FRAC_ONE - 1;
	iop2 = 1;
	fop2 = 0;

	argv++;

	if (argc-- > 1)
	{
		nbits = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		minindex = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		maxindex = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		minfrac = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		maxfrac = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		iop2 = atoi(*argv++);
	}
	if (argc-- > 1)
	{
		fop2 = atoi(*argv++);
	}

	if (maxfrac >= FRAC_ONE)
	{
		fprintf(stderr, "main: max fraction must be < %lu\n", FRAC_ONE);
		return 1;
	}

	printf("nbits = %d\n", nbits);
	printf("minindex = %lu, maxindex = %lu\n", minindex, maxindex);
	printf("minfrac = %lu, maxfrac = %lu\n", minfrac, maxfrac);
	printf("iop2 = %ld, fop2 = %lu\n", iop2, fop2);
	
	/* initialise */
	result = frac_log2_init(nbits);
	if (result < 0)
	{
		fprintf(stderr, "main: frac_log2_init(%d) failed\n", nbits);
		return 1;
	}


	/* sum */
	result = test_sum(minindex, maxindex, minfrac, maxfrac, iop2, fop2);
	if (result < 0)
	{
		fprintf(stderr, "main: test_sum failed\n");
		return 1;
	}

	/* mul */
	result = test_mul(minindex, maxindex, minfrac, maxfrac, iop2, fop2);
	if (result < 0)
	{
		fprintf(stderr, "main: test_mul failed\n");
		return 1;
	}

	/* div */
	result = test_div(minindex, maxindex, minfrac, maxfrac, iop2, fop2);
	if (result < 0)
	{
		fprintf(stderr, "main: test_div failed\n");
		return 1;
	}

	/* log2 */
	result = test_log2(minindex+1, maxindex, nbits);
	if (result < 0)
	{
		fprintf(stderr, "main: test_log2 failed\n");
		return 1;
	}

	/* sqrt */
	result = test_sqrt(minindex, maxindex, nbits); 
	if (result < 0)
	{
		fprintf(stderr, "main: test_sqrt failed\n");
		return 1;
	}

	/* pow2 */
	result = test_pow2(minindex, maxindex);
	if (result < 0)
	{
		fprintf(stderr, "main: test_pow2 failed\n");
		return 1;
	}

	return 0;
}

