/*! 
\file distfunc.h
\brief Interface to distance function

The distance functions compute distance
between two vectors with different
metrics: Euclidean distance,
Manhattan distance, Maximum distance.
These metrics are induced by the common vector
space norms
\f$||\cdot ||_2\f$, 
\f$||\cdot ||_1\f$, and 
\f$||\cdot ||_\infty\f$.\n
$Id: distfunc.h,v 1.1 2003/11/20 13:42:31 ekarpov Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef __DISTFUNC_H__
#define __DISTFUNC_H__

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------*/

/*!
\brief Distance function type

All distance functions must be able to
compute distance of two vectors based
on the vector data and length of the vectors.
\param fv1 Pointer to data of vectors 1
\param fv2 Pointer to data of vectors 2
\param params This should be pointer to
integer describing the length of input vectors.
\return The computed distance.
 */
typedef double (*DISTFUNC) (
	double * fv1,
	double * fv2,
    int dimension,
	void* params) ;

/*======================== Functions ==============================*/

/* distance between two vectors */
double dfEuclidean(double * fv1, double * fv2, 
                   int dimension, void* params) ;
double dfManhattan(double * fv1, double * fv2, 
                   int dimension, void* params) ;
double dfMaximum(double * fv1, double * fv2, 
                 int dimension, void* params) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __DISTFUNC_H__ */
