/*! 
\file 
\brief Metric space indexing

Common framework to work with indexes. Can be used to create 
indexes and search trough the data. Supposed to work with 
vector sets only. Should be used instead of direct usage of 
indexing algorithms.

\date 05 November 2003
\author Evgeny Karpov
*/

/*
    $Id: msindex.h,v 1.3 2003/11/20 13:46:12 ekarpov Exp $
*/

#ifndef __MSINDEX_H__
#define __MSINDEX_H__

/*-----------------------------------------------------------------*/

#include "fvector.h"
#include "distfunc.h"

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Structures =============================*/

typedef enum {

    MSINDEX_ALGTYPE_VPT = 1 /* Vantage-Point Tree */

} MSIndexType ;

/*======================== Structures =============================*/

typedef void * HMSIDX ;    /* metric space indexer */
typedef void * HMSINDEX ;  /* indexes */

/* TODO: CHANGE VPTDISTFUNC TO GLOBAL TYPE */

/*======================== Functions ==============================*/

/* creation/deletion */
HMSIDX msidxCreate(MSIndexType type) ;
void msidxDestroy(HMSIDX msidx) ;
void msidxDestroyIndexes(HMSIDX msidx,void * indexes) ;

/* index building */
HMSINDEX msidxTrain(HMSIDX msidx,HFSET test_set,
                    DISTFUNC dist_func, void * dfunc_params) ;

/* search */
int msidxSearch(HMSIDX msidx, HMSINDEX indexes, HFSET search_set,
                double * element, double radius, int * ret_arr) ;

/* nearest neighbour search */
int msidxNNSearch(HMSIDX msidx, HMSINDEX indexes, HFSET search_set,
                  double * element, double * ret_dist) ;

/* parameters */
MSIndexType msidxGetType(HMSIDX msidx) ;

/* save/load */
HMSINDEX msidxLoad(MSIndexType type, char * file_name) ;
int msidxSave(MSIndexType type, HMSINDEX indexes,char * file_name) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __MSINDEX_H__ */

