/*!
\file dsp.h
\brief Digital signal processing interface

Interface for working with several
well-known digital signal processing algorithms\n
$Id: dsp.h,v 1.2 2003/12/18 10:17:18 juhani Exp $
\todo The logarithms and square roots and all math functions
values must be scaled later after a proper analysis is done
about how they should be scaled when using INT16 or INT32;
this may also change the interface, if the scaling will
be adjustable during run time.
\date 2003
\author Evgeny Karpov
*/

#ifndef DSP_H
#define DSP_H


#include <fixmath.h>


#if defined(__cplusplus)
extern "C" {
#endif


/*======================== Functions ==============================*/

/*
Inner product
 */
INT32 dspInnerProduct(
	INT16 * factor1,
	INT16 * factor2,
	int size);

/*
Pointwise multiplication
 */
int dspMultiply(
        INT16 * factor1,
        INT16 * factor2,
        INT32 * output,
        int size);

/*
Convolution
 */
int dspConvolution(
	INT16 * input,
	int input_size,
	INT16 * kernel,
	int kernel_size,
	INT32 * output) ;

/*
DC Removal
 */
int dspDCRemove(
	INT16 * input,
	INT16 * output,
	int size,
	void * params) ;


/*
First Order Difference Filtering
 */
int dspHighEmphFilter(
	INT16 * input,
	INT16 * output,
	int size,
	void * params) ;


/*
Number of Zero Crossings
 */
int dspZCR(
	INT16 * input,
	int size) ;


/*
Vector Energy
 */
long dspEnergy(
	INT16 * input,
	int size) ;


/*
Power of a Finite Digital Signal
 */
long dspPower(
	INT16 * input,
	int size) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* DSP_H */
