/*! 
\file fftdct.h
\brief A generic FFT and DCT library interface,
used by MFCC module.

This is an interface for FFT and DCT.\n
$Id: fftdct.h,v 1.3 2003/12/29 12:06:07 juhani Exp $
\todo The logarithms and square roots and all math functions
values must be scaled later after a proper analysis is done
about how they should be scaled when using INT16 or INT32;
this may also change the interface, if the scaling will
be adjustable during run time.
\date 2003/11
\author Juhani Saastamoinen
*/
#ifndef FFTDCT_H
#define FFTDCT_H


#include <fixmath.h>
#include <shortfft.h>


#define DCT_SCALE 32767


typedef struct
{
	int input_size;
	void* fft_plan;
} FFTProcessor;


typedef struct
{
	int input_size;
	int spectrum_size;
	INT16** cos_functions;
} DCTProcessor;


FFTProcessor* fftCreate(int input_size);
void fftDestroy(FFTProcessor* fft);
int fftGetInputSize(FFTProcessor* fft);
int fftCompute(FFTProcessor* fft, INT16* input, INT32* spectrum);


DCTProcessor* dctCreate(int input_size, int spectrum_size);
void dctDestroy(DCTProcessor* fft);
int dctGetInputSize(DCTProcessor* fft);
int dctGetSpectrumSize(DCTProcessor* fft);
int dctCompute(DCTProcessor* fft, INT16* input, INT32* spectrum);


#endif /* #ifndef FFTDCT_H */

