/*!
\file filterbank.h
\brief Filter bank interface

Interface for working with bank of linear filters
operating with magnitude spectra
in the frequency domain.\n
$Id: filterbank.h,v 1.6 2004/01/02 16:09:54 juhani Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef FILTERBANK_H
#define FILTERBANK_H


#include <fixmath.h>


#if defined(__cplusplus)
extern "C" {
#endif


/*-----------------------------------------------------------------*/


/*!
\brief ???
\todo Document this
 */
#define FILTERBANK_FILTERS_AUTO         0


/*-----------------------------------------------------------------*/


/*! 
\brief Abstract filter bank handle type

\sa FilterBank
*/
typedef void * HFBANK ;


/*======================== Functions ==============================*/

/* creation/deletion */
HFBANK fbCreate(int filters,int average) ;
void fbDestroy(HFBANK fbank) ;

/* filter management */
int fbAddFilter(HFBANK fbank, int begin, int end, INT16 * coeff) ;
int fbInsertFilter(
		HFBANK fbank, int begin, int end, INT16 * coeff, int position) ;
int fbRemoveFilter(HFBANK fbank,int filter) ;

int fbGetFilterBegin(HFBANK fbank,int filter) ;
int fbGetFilterEnd(HFBANK fbank,int filter) ;
int fbGetFilterLength(HFBANK fbank,int filter) ;
INT16 * fbGetFilterCoeff(HFBANK fbank,int filter) ;

int fbGetSize(HFBANK fbank) ;

/* applying filter bank */
int fbApply(HFBANK fbank,
            INT16 * input, int input_size,
            INT32 * output, int output_size) ;
INT32 fbApplyFilter(HFBANK fbank,
                    INT16 * input, int input_size, int filter) ;

/* print filter bank to stdout */
int fbPrint(FILE* f, HFBANK fbank);
int fbPrintFilter(FILE* f, int begin, int end, INT16* coeff);

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* FILTERBANK_H */

