/*!
\file winfunc.h
\brief Signal windowing functions interface

Interface for computing several
standard signal windowing functions\n
$Id: winfunc.h,v 1.2 2003/12/18 10:17:18 juhani Exp $
\todo The logarithms and square roots and all math functions
values must be scaled later after a proper analysis is done
about how they should be scaled when using INT16 or INT32;
this may also change the interface, if the scaling will
be adjustable during run time.
\date 2003
\author Evgeny Karpov
*/

#ifndef WINFUNC_H
#define WINFUNC_H


#include <fixmath.h>

    
#if defined(__cplusplus)
extern "C" {
#endif


#define WINDOWING_SCALE 32767


/*-----------------------------------------------------------------*/


/*!
\brief Window function pointer type

A window function always takes two parameters:
pointer to output data and window vector length.
Different window functions evaluate different vectors.
*/
typedef int (*WINFUNC) (INT16 * coeff,int win_size) ;


/*======================== Functions ==============================*/


/*
Hamming window
 */
int wfHamming(
	INT16 * coeff,
	int win_size) ;


/*
Bartlett window
 */
int wfBarlett(
	INT16 * coeff,
	int win_size) ;


/*
Hanning window
 */
int wfHanning(
	INT16 * coeff,
	int win_size) ;


/*
Blackman window
 */
int wfBlackman(
    INT16 * coeff,
    int win_size) ;


/*
Gaussian window
 */
int wfGaussian(
    INT16 * coeff,
    int win_size) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* WINFUNC_H */
