/*! 
\file distfunc.h
\brief Interface to distance function

The distance functions compute distance
between two vectors with different
metrics: Euclidean distance,
Manhattan distance, Maximum distance.
These metrics are induced by the common vector
space norms
\f$||\cdot ||_2\f$, 
\f$||\cdot ||_1\f$, and 
\f$||\cdot ||_\infty\f$.\n
$Id: distfunc.h,v 1.2 2003/12/18 10:15:25 juhani Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef __DISTFUNC_H__
#define __DISTFUNC_H__


#include <fixmath.h>


#if defined(__cplusplus)
extern "C" {
#endif


/*-----------------------------------------------------------------*/

/*!
\brief Distance function type

All distance functions must be able to
compute distance of two vectors based
on the vector data and length of the vectors.
\param fv1 Pointer to data of vectors 1
\param fv2 Pointer to data of vectors 2
\param params This should be pointer to
integer describing the length of input vectors.
\return The computed distance.
 */
typedef INT32 (*DISTFUNC) (
	INT16 * fv1,
	INT16 * fv2,
    int dimension,
	void* params) ;

/*======================== Functions ==============================*/

/* distance between two vectors */
INT32 dfEuclidean(INT32 * fv1, INT32 * fv2, 
                   int dimension, void* params) ;
INT32 dfManhattan(INT32 * fv1, INT32 * fv2, 
                   int dimension, void* params) ;
INT32 dfMaximum(INT32 * fv1, INT32 * fv2, 
                 int dimension, void* params) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __DISTFUNC_H__ */
