/*! 
\file fvector.h
\brief Feature vector set interface

The feature vector set modules has
all the functionality neede for working
with sets of feature vectors.\n
$Id: fvector.h,v 1.4 2004/01/02 16:08:05 juhani Exp $
\date 2003
\author Evgen Karpov
*/

#ifndef FVECTOR_H
#define FVECTOR_H


#include <fixmath.h>


#if defined(__cplusplus)
extern "C" {
#endif


/*======================== Structures =============================*/

/*! 
\brief Set of feature vectors

A data structure for storing set of vectors.
\sa HFSET
*/
typedef struct {

    int vector_number ;  /*!< number of last vector added */
    int array_size ;     /*!< maximum number of vectors */

    int vector_size ;    /*!< vector size */

    INT32 ** vectors ; /*!< vector data */

} FVSet ;

/*======================== Structures =============================*/

/*! Abstract feature vector set handle */
typedef FVSet * HFSET ;

/*-----------------------------------------------------------------*/

/*! Macro for easier access of given vector */
#define FVS_GET_ELEMENT(fvs,num)    ((FVSet*)(fvs))->vectors[num]

/*======================== Functions ==============================*/

/* feature vector creation/deletion */
INT32 * fvCreate(int size) ;
void fvDestroy(INT32 * fv) ;

/* feature vector set creation/deletion */
HFSET fvsCreate(int size,int vector_size) ;
void fvsDestroy(HFSET fvs) ;

/* working with set */
int fvsAddVector(HFSET fvs,INT32 * fv) ;
int fvsAddVectorCopy(HFSET fvs,INT32 * fv) ;
int fvsRemoveVector(HFSET fvs,int num) ;
int fvsInsertVector(HFSET fvs,INT32 * fv,int num) ;
int fvsReplaceVector(HFSET fvs,INT32 * fv,int num) ;
int fvsGetSize(HFSET fvs) ;
int fvsGetVectorSize(HFSET fvs) ;

/* getting vectors from set */
INT32 * fvsGetVector(HFSET fvs,int num) ;
INT32 * fvsCopyVector(HFSET fvs,int num) ;

/* set loading/storing */
HFSET fvsLoad(char * file_name) ;
int fvsSave(HFSET fvs,char * file_name) ;

/* set merging */
HFSET fvsMerge(HFSET fvs1,HFSET fvs2) ;

/* working with internal data */
int fvsSetSize(HFSET fvs,int new_size) ;
int fvsSetVectorSize(HFSET fvs,int new_size) ;
void * fvsGetVectorBuff(HFSET fvs) ;

/* print a vector of a vector set */
int fvsPrintVector(FILE* f, HFSET fvs, int num);

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* FVECTOR_H */

