/*! 
\file 
\brief Vantage-Point Tree

Fast nearest neighbour search using VPT

\date 05 November 2003
\author Evgeny Karpov
*/

/*
    $Id: vptree.h,v 1.1 2003/11/30 20:39:30 juhani Exp $
*/

#ifndef __VPTREE_H__
#define __VPTREE_H__

/*-----------------------------------------------------------------*/

#include <stdio.h>

#include "fvector.h"
#include "distfunc.h"

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------*/

/*! handle to vpt-index */
typedef void * HVPTREE ;

/*======================== Functions ==============================*/

/* creation/deletion */
HVPTREE vptCreate(HFSET index_set,DISTFUNC dist_func,
                  void * dfunc_params) ;

void vptDestroy(HVPTREE vpt) ;

/* nearest neighbour search */
int vptNNSearch(HVPTREE vpt, HFSET index_set, 
                double * search_vector, double * ret_dist);

/* search */
int vptSearch(HVPTREE vpt, HFSET index_set,
              double * search_vector, double radius, int * ret_arr);

/* load/save */
HVPTREE vptLoad(FILE * file) ;
int vptSave(HVPTREE vpt,FILE * file) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __VPTREE_H__ */

