/*-----------------------------------------------------------------*/
/* Module : Configuration files                                    */
/*                                                                 */
/* Description : Interface for working with configuration files    */
/*                                                                 */
/*-----------------------------------------------------------------*/

/*
    $Id: cfgfile.c,v 1.1 2003/12/04 11:40:45 villeh Exp $
*/

#include "cfgfile.h"

#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

/*======================== Structures =============================*/

typedef struct {

    char * name ;
    int size ;

    char ** params ;
    char ** values ;

} CfgSection ;

typedef struct {

    CfgFileMode mode ;

    CfgSection * sections ;

    int size ;

    int curr_section ;
    int curr_param ;

    FILE * pfile ;

} CfgFile ;

/* TODO: IMPLEMENT SIMPLE HASHING FOR FAST SEARCHING */

/*======================== Structures =============================*/

void free_sections(CfgSection * sections,int size) ;
int read_config_file(CfgFile * cfp) ;

/*======================== Functions ==============================*/

HCFGFILE cfOpen(char * file_name,CfgFileMode mode) 
{
    CfgFile * cfp ;

    if (file_name==NULL)
    {
        errno = EINVAL ;
        return NULL ;
    }

    if ((cfp = malloc(sizeof(CfgFile)))==NULL)
    {
        errno = ENOMEM ;
        return NULL ;
    }

    cfp->mode = mode ;
    cfp->sections = NULL ;
    cfp->size = 0 ;

    cfp->curr_section = 0 ;
    cfp->curr_param = 0 ;

    switch(mode)
    {
    case CFILE_CHANGE:
        cfp->pfile = fopen(file_name,"r+") ;
        break ;

    case CFILE_READ:
        cfp->pfile = fopen(file_name,"r") ;
        break ;

    case CFILE_WRITE:
        cfp->pfile = fopen(file_name,"w") ;
        break ;

    default:

        errno = EINVAL ;
        free(cfp) ;
        return NULL ;
    }

    if (cfp->pfile == NULL)
    {
        /* errno = ??? ferror(cfp->pfile) ??? */
        free(cfp) ;
        return NULL ;
    }

    if (mode == CFILE_WRITE)
    {
    } else {
        /* read file */
        if (read_config_file(cfp)) return (HCFGFILE) cfp ;
    }

    free(cfp) ;
    return NULL ;
}

void cfClose(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) return ;

    free_sections(cfp->sections,cfp->size) ;

    fclose(cfp->pfile) ;
    free(cfp) ;
}

/*-----------------------------------------------------------------*/

int cfSize(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    return cfp->size ;
}

int cfSelectSection(HCFGFILE cf,char * section_name) 
{
    CfgFile * cfp ;
    char * pleft, * pright ;
    int i ;

    if ((cfp = (CfgFile*)cf)==NULL || section_name == NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    for (i=0;i<cfp->size;i++)
    {
        pleft = cfp->sections[i].name ;
        pright = section_name ;

        /* case insensitive comparison of two strings */
        while (*pleft != 0 && *pright != 0)
        {
            if (tolower(*pleft) != tolower(*pright)) break ;

            pleft++ ;
            pright++ ;
        }

        if (*pleft == *pright && *pleft == 0)
        {
            /* first occurence found => select this section */
            cfp->curr_section = i ;
            return 1 ;
        }
    }

    return 0 ;
}

int cfCreateSection(HCFGFILE cf,char * section_name) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

/*-----------------------------------------------------------------*/

int cfSectionSize(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size > 0)
        return cfp->sections[cfp->curr_section].size ;

    return 0 ;
}

int cfSelectParam(HCFGFILE cf,char * param_name) 
{
    CfgFile * cfp ;
    CfgSection * psection ;
    char * pleft, * pright ;
    int i ;

    if ((cfp = (CfgFile*)cf)==NULL || param_name == NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size == 0) return 0 ;

    psection = cfp->sections + cfp->curr_section ;

    for (i=0;i<psection->size;i++)
    {
        pleft = psection->params[i] ;
        pright = param_name ;

        /* case insensitive comparison of two strings */
        while (*pleft != 0 && *pright != 0)
        {
            if (tolower(*pleft) != tolower(*pright)) break ;

            pleft++ ;
            pright++ ;
        }

        if (*pleft == *pright && *pleft == 0)
        {
            /* first occurence found => select this parameter */
            cfp->curr_param = i ;
            return 1 ;
        }
    }

    return 0 ;
}

int cfCreateParam(HCFGFILE cf,char * param_name) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

/*-----------------------------------------------------------------*/

char * cfGetTextValue(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size > 0 && cfp->sections[cfp->curr_section].size > 0)
        return cfp->sections[cfp->curr_section].values[cfp->curr_param] ;

    return NULL ;
}

int cfGetBoolValue(HCFGFILE cf) 
{
    CfgFile * cfp ;
    char * pfind ;
    int i ;

    char yes[] = {'y','e','s'};

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size == 0 || cfp->sections[cfp->curr_section].size == 0) return 0 ;

    pfind = cfp->sections[cfp->curr_section].values[cfp->curr_param] ;

    if (strlen(pfind) != 3) return 0 ;

    /* compare with 'yes' */
    for (i=0;i<3;i++)
    {
        if (tolower(*pfind) != yes[i]) return 0 ;
        pfind++ ;
    }

    if (*pfind==0) return 1 ;

    return 0 ;
}

int cfGetIntValue(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size > 0 && cfp->sections[cfp->curr_section].size > 0)
        return atoi(cfp->sections[cfp->curr_section].values[cfp->curr_param]) ;

    return 0 ;
}

double cfGetDoubleValue(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    if (cfp->size > 0 && cfp->sections[cfp->curr_section].size > 0)
        return atof(cfp->sections[cfp->curr_section].values[cfp->curr_param]) ;

    return 0.0 ;
}

/*-----------------------------------------------------------------*/

char * cfCopyTextValue(HCFGFILE cf) 
{
    char * val, * ret_val ;

    if ((val=cfGetTextValue(cf)))
    {
        if ((ret_val = malloc(sizeof(char)*(strlen(val)+1)))==NULL)
        {
            errno = ENOMEM ;
            return NULL ;
        }

        strcpy(ret_val,val) ;

        return ret_val ;
    }

    return NULL ;
}

/*-----------------------------------------------------------------*/

int cfSetTextValue(HCFGFILE cf,char * value) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

int cfSetBoolValue(HCFGFILE cf,int value) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

int cfSetIntValue(HCFGFILE cf,int value) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

int cfSetDoubleValue(HCFGFILE cf,double value) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

/*-----------------------------------------------------------------*/

int cfSelectNextSection(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

int cfSelectNextParam(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return 0 ;
}

char * cfGetParam(HCFGFILE cf) 
{
    CfgFile * cfp ;

    if ((cfp = (CfgFile*)cf)==NULL) 
    {
        errno = EINVAL ;
        return 0 ;
    }

    /* NOT IMPLEMENTED YET !!! */

    return NULL ;
}

/*-----------------------------------------------------------------*/

void free_sections(CfgSection * sections,int size)
{
    int i,j ;

    for (i=0;i<size;i++)
    {
        for (j=0;j<sections[i].size;j++)
        {
            free(sections[i].params[j]) ;
            free(sections[i].values[j]) ;
        }

        free(sections[i].params) ;
        free(sections[i].values) ;
        free(sections[i].name) ;
    }

    free(sections) ;
}

int read_config_file(CfgFile * cfp) 
{
    CfgSection * psection ;

    char * pbuff, * ptmp, * pfind, * pval ;
    char ** ptmppar ;

    int buff_size, pos, el ;

    char ch ;

    int line_count ;

    buff_size = 100 ; /* initial buffer size */

    /* allocate initial buffer */
    if ((pbuff = malloc(buff_size))==NULL)
    {
        errno = ENOMEM ;
        return 0 ;
    }

    psection = NULL ; line_count = 0 ;

    while (feof(cfp->pfile)==0)
    {
        /* === read line from file === */

        pos = 0 ; el = 0 ;

        while (el==0)
        {
            ch = getc(cfp->pfile) ;

            if (ch == '\n' || ch == '\r' || ch == EOF || ch == ';')
            {
                el = 1 ; 

                /* skip comments */
                if (ch == ';')
                    while (ch!='\r' && ch!='\n' && ch!=EOF) ch=getc(cfp->pfile) ;

                /* Remove all '\r' and '\n' */
                while ((ch=='\r' || ch=='\n') && ch!=EOF) ch=getc(cfp->pfile) ;
                if (ch!=EOF) ungetc(ch,cfp->pfile) ; /* Return back one character */

                line_count++ ;
                pbuff[pos] = 0; /* put zero at the end */

            } else {

                if (pos>=buff_size)
                {
                    /* enlarge temporary buffer */
                    buff_size = buff_size * 2 ;

                    if ((ptmp = realloc(pbuff,buff_size))==NULL)
                    {
                        errno = ENOMEM ;
                        free(pbuff) ;
                        free_sections(cfp->sections,cfp->size) ;
                        return 0 ;
                    }

                    pbuff = ptmp ;
                }

                pbuff[pos++] = ch ; /* copy character to buffer */
            }
        }

        if (pos == 0) continue ; /* empty or comment line */

        /* === parse read line === */

        pfind = pbuff ;
        /* skip all spaces */
        while ((*pfind == ' ' || *pfind == '\t') && *pfind != 0) pfind++ ;

        if (*pfind == 0) continue ; /* skip empty line */

        if (*pfind == '[')
        {
            /* === new section === */

            pfind++ ; /* skip [ */

            /* remove spaces */
            while ((*pfind == ' ' || *pfind == '\t') && *pfind != 0 && *pfind != ']') pfind++ ;

            /* find the end */
            for (ptmp=pfind;*ptmp != ']' && *ptmp != 0;ptmp++) ;

            if (*ptmp == 0 && *pfind != 0 && *pfind != ']') 
                printf("cfgfile warning (line %i): closing bracket \']\' not found\n",line_count) ;
            
            /* remove spaces at the end */
            while ((*(ptmp-1)==' ' || *(ptmp-1)=='\t') && ptmp > pfind) *(ptmp--)=0 ;

            *ptmp = 0 ;

            if (*pfind == 0 || *pfind == ']' || ptmp == pfind) /* error - section name is empty */
            {                
                /* errno = ??? (invalid format) */
                free(pbuff) ;
                free_sections(cfp->sections,cfp->size) ;
                printf("cfgfile parsing error (line %i): Section name is empty\n",line_count) ;
                return 0 ;
            }

            if ((psection = realloc(cfp->sections,sizeof(CfgSection)*(cfp->size+1))) != NULL)
            {
                cfp->sections = psection ;
                psection = cfp->sections + cfp->size ;
                psection->name = malloc(sizeof(char)*(ptmp-pfind+1)) ;
            }

            if (psection == NULL || psection->name == NULL)
            {
                errno = ENOMEM ;
                free(pbuff) ;
                if (psection) free(psection->name) ;
                free(psection) ;
                free_sections(cfp->sections,cfp->size) ;
                return 0 ;
            }

            /* copy section name */
            strcpy(psection->name,pfind) ;

            psection->size = 0 ;
            psection->params = NULL ;
            psection->values = NULL ;
            
            cfp->size++ ;

        } else {

            /* === new value === */

            if (psection == NULL) /* error - value not in section */
            {                
                /* errno = ??? (invalid format) */
                free(pbuff) ;
                free_sections(cfp->sections,cfp->size) ;
                printf("cfgfile parsing error (line %i): Value not in section\n",line_count) ;
                return 0 ;
            }

            ptmp = pfind ;

            while (*ptmp != '=' && *ptmp != 0) ptmp ++ ;

            if (*ptmp == 0 || ptmp == pfind) 
            {
                printf("cfgfile warning (line %i): value not set (parameter ignored)\n",line_count) ;
                continue ;
            }

            pval = ptmp + 1 ;

            /* remove spaces at the end */
            while (*(ptmp-1)==' ' || *(ptmp-1)=='\t') *(ptmp--)=0 ;

            *ptmp = 0 ;

            if ((ptmppar = realloc(psection->params,sizeof(char*)*(psection->size+1)))!=NULL)
            {
                psection->params = ptmppar ;
                if ((psection->params[psection->size] = malloc(sizeof(char)*(strlen(pfind)+1)))!=NULL)
                    strcpy(psection->params[psection->size],pfind) ;
            }

            if (ptmppar == NULL || ptmppar[psection->size] == NULL)
            {                
                errno = ENOMEM ;
                if (ptmppar) free(ptmppar[psection->size]) ;
                free(ptmppar) ;
                free_sections(cfp->sections,cfp->size) ;
                return 0 ;
            }

            /* remove spaces */
            while ((*pval == ' ' || *pval == '\t') && *pval != 0) pval++ ;

            /* find the end */
            for (ptmp=pval;*ptmp != 0;ptmp++) ;

            if (*pval == 0) /* empty value */
            {
                printf("cfgfile warning (line %i): value not set (parameter ignored)\n",line_count) ;
                if (ptmppar) free(ptmppar[psection->size]) ;
                continue ;

            } else for (ptmp--;(*ptmp == ' ' || *ptmp == '\t');*(ptmp--)=0) ; /* remove spaces at the end */

            if ((ptmppar = realloc(psection->values,sizeof(char*)*(psection->size+1)))!=NULL)
            {
                psection->values = ptmppar ;
                if ((psection->values[psection->size] = malloc(strlen(pval)+1))!=NULL)
                    strcpy(psection->values[psection->size],pval) ;
            }

            if (ptmppar == NULL || ptmppar[psection->size] == NULL)
            {                
                errno = ENOMEM ;
                if (ptmppar) free(ptmppar[psection->size]) ;
                free(ptmppar) ;
                free(psection->params[psection->size]) ;
                free_sections(cfp->sections,cfp->size) ;
                return 0 ;
            }


            psection->size++ ;
        }
    }

    free(pbuff) ;

    return 1 ;
}

/*======================== Functions ==============================*/

