/*! 
\file 
\brief Speaker database handling

Provides mechanism for working with simple speaker database.
Speaker database in this realization consits of simple ASCII
file, one record per a row, fields are separated by comma.

$Id: speakerdb.h,v 1.6 2003/12/04 10:36:40 ekarpov Exp $

\date 03 November 2003
\author Evgeny Karpov

  $Id: speakerdb.h,v 1.6 2003/12/04 10:36:40 ekarpov Exp $

*/

#ifndef __SPEAKERDB_H__
#define __SPEAKERDB_H__

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Structures =============================*/

/*! Speaker database open modes */
typedef enum {

    SDB_OPEN_READ_ONLY,     /*!< Open read-only */
    SDB_OPEN_CHANGE,        /*!< Open for change */
    SDB_OPEN_NEW            /*!< Open new */

} SdbOpenMode ;

/*======================== Structures =============================*/

/*! Abstract function type for model loading function */
typedef void * (*SDBMLOAD) (char * file_name,void * params) ;

/*-----------------------------------------------------------------*/

/*! Abstract handle for speaker database */
typedef void * HSDB ;

/*======================== Functions ==============================*/

/* open/close */
HSDB sdbOpen(char * file_name,SdbOpenMode mode) ;
void sdbClose(HSDB sdb) ;

/* load model for selected speaker */
void * sdbLoadModel(HSDB sdb,SDBMLOAD load_func,void * load_params) ;

/* save database */
int sdbSave(HSDB sdb) ;
int sdbSaveAs(HSDB sdb, char * file_name) ;

/* record selection function */
int sdbSelect(HSDB sdb,int num) ;
int sdbSelectNext(HSDB sdb) ;
int sdbReset(HSDB sdb) ;

/* reading raw data */
int sdbSize(HSDB sdb) ;
int sdbGetIntValue(HSDB sdb,int field) ;
char * sdbGetTextValue(HSDB sdb,int field) ;

/* changing raw data */
int sdbSetIntValue(HSDB sdb,int field,int value) ;
int sdbSetTextValue(HSDB sdb,int field,char * value) ;

/* reading speakers */
int sdbGetSpeakerID(HSDB sdb) ;
char * sdbGetSpeakerName(HSDB sdb) ;
char * sdbGetSpeakerModelPath(HSDB sdb) ;

/* changind speaker parameters */
int sdbSetSpeakerID(HSDB sdb, int ID) ;
int sdbSetSpeakerName(HSDB sdb,char * speaker_name) ;
int sdbSetSpeakerModelPath(HSDB sdb,char * model_path) ;

/* insert and remove records */
int sdbAddRecord(HSDB sdb,int field_num) ;
int sdbRemoveRecord(HSDB sdb) ;

/* insert and remove speakers */
int sdbAddSpeaker(HSDB sdb,int id,char * name,char * model_path) ;
int sdbRemoveSpeaker(HSDB sdb,int speaker_id) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __SPEAKERDB_H__ */

