/*! 
\file errfunc.h
\brief Interfaces to clustering error functions

 $Id: errfunc.h,v 1.4 2003/11/20 11:36:29 ekarpov Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef __ERRFUNC_H__
#define __ERRFUNC_H__

/*-----------------------------------------------------------------*/

#include "fvector.h"
#include "distfunc.h"

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------*/

/*!
\brief Clustering (???) error function type

All clustering error measurement implementations
should have similar prototype for computing
distance of two feature vector sets. ???
\param cb Codebook, a feature vector set
\param test_set Vector set tested against codebook \p cb
\param dist_func Distance function used in computations
\return Error value ???
\todo Verify the dodumentation
\sa FVSet
\sa DISTFUNC
*/
typedef double (*ERRFUNC) (
	HFSET cb,
	HFSET test_set,
	DISTFUNC dist_func,
    void * dfunc_params) ;

/*======================== Functions ==============================*/

/* mean squared error */
double errMSE(HFSET codebook,HFSET test_set,DISTFUNC dist_func,
              void * dfunc_params) ;

/* total squared error */
double errTSE(HFSET codebook,HFSET test_set,DISTFUNC dist_func,
              void * dfunc_params) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __ERRFUNC_H__ */
