/*! 
\file gla.h
\brief The Generalized Lloyd Algorithm interface

$Id: gla.h,v 1.6 2003/11/20 11:42:36 ekarpov Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef __GLA_H__
#define __GLA_H__

/*-----------------------------------------------------------------*/

#include "fvector.h"
#include "distfunc.h"
#include "errfunc.h"
#include "idxerrfunc.h"

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif


/*======================== Structures =============================*/


/*! 
\brief Data and function pointer needed by GLA algorithm
\sa DISTFUNC
\sa ERRFUNC
\sa IDXERRFUNC
*/
typedef struct {

    int max_iteration ; /*!< maximum number of GLA iterations */
    double threshold ;  /*!< convergence criterion
                             for stopping the iteration */

    DISTFUNC dist_func ; /*!< distance function used for partitioning */
    void * dfunc_params ; /*!< distance function parameters */

    ERRFUNC  err_func ;      /*!< normal error function */
    IDXERRFUNC idxerr_func ; /*!< fast error function */

    int flags ;  /*!< parameters */

} GLAParams ;


/*======================== Structures =============================*/


/* flags */
#define GLA_DISTANCE_FUNC       1
#define GLA_ERROR_FUNC          2
#define GLA_USE_INDEXING        4
#define GLA_FERROR_FUNC         8


/*======================== Functions ==============================*/


/* create codebook */
int glaRun(HFSET init_cb, HFSET fvs,
           GLAParams * params, double * ret_err) ;


/*======================== Functions ==============================*/


#if defined(__cplusplus)
}
#endif

#endif /* __GLA_H__ */

