/*! 
\file idxerrfunc.h
\brief Interface to clustering error functions that use indexing

$Id: idxerrfunc.h,v 1.8 2003/11/10 20:35:06 juhani Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef __IDXERRFUNC_H__
#define __IDXERRFUNC_H__

/*-----------------------------------------------------------------*/

#include <msindex.h>


#include "fvector.h"
#include "distfunc.h"

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------*/

/*!
\brief Error function type when indexing is used

All error functions that utilize indexing
should have same parameters and return type
\param codebook Input centroid vector set againts which
the error is computed
\param test_set Input vector set, whose clustering
error against codebook cb is measured
\param msidx Metric space indexer
\param indexes Metric space index
\return The computed clustering error value
\sa FVSet
\sa MSIndex
\sa DISTFUNC
*/typedef double (*IDXERRFUNC) (
    HFSET codebook, HFSET test_set, HMSIDX msidx, HMSINDEX indexes) ;

/*======================== Functions ==============================*/

/* mean squared error */
double idxMSE(
    HFSET codebook, HFSET test_set, HMSIDX msidx, HMSINDEX indexes) ;

/* total squared error */
double idxTSE(
    HFSET codebook, HFSET test_set, HMSIDX msidx, HMSINDEX indexes) ;

/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* __IDXERRFUNC_H__ */

