/*! 
\file pq.h
\brief Pre-quantization interface

Algorithms for quantization of input data.\n
$Id: pq.h,v 1.6 2003/11/13 16:41:03 juhani Exp $
\todo Explain in more detail:
What kind of data ???
Where is this used, why, how, ...
\date 2003
\author Evgeny Karpov
*/

#ifndef PQ_H
#define PQ_H

#include <fvector.h>

/*-----------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

/*======================== Structures =============================*/

/*!
\brief Pre-quantization algorithm types
 */
typedef enum {

    PQTYPE_RANDOM,   /*!< use only random vectors */
    PQTYPE_AVERAGE,  /*!< average input data */
    PQTYPE_SUBSAMPL, /*!< subsampling */
    PQTYPE_CODEBOOK  /*!< create codebook of test data */

} PQALG ;

/*======================== Structures =============================*/

/*!
\brief Data reduction parameters
*/
typedef struct
{
    PQALG algorithm; /*!< algorithm used */
    union QuantizationParam
    {
        int interval_size;      /*!< Interval size used for interval
                                     averaging, random picking from
                                     interval, and subsampling */
        int number_of_clusters; /*!< number of clusters for
                                     codebook based reduction */
    } output_size_spec;
} PQParams;


/*======================== Functions ==============================*/


/* quantization */
HFSET pqQuantize(PQParams pq, HFSET test_set) ;


/*======================== Functions ==============================*/

#if defined(__cplusplus)
}
#endif

#endif /* PQ_H */

