/*! 
\file spruning.h
\brief Interface to speaker pruning functions

This module defines a framework for speaker pruning.\n
$Id: spruning.h,v 1.6 2003/11/26 08:01:27 ekarpov Exp $
\date 2003
\author Evgeny Karpov
*/

#ifndef SPRUNING_H
#define SPRUNING_H

#include <modelset.h>
#include <fvector.h>

#include "pq.h"

#include "vq.h" /* REMOVE THIS !!! */

/*======================== Structures =============================*/

typedef struct {

    int init_interval ;  /* initial interval size */
    int max_iteration ;  /* maximum iterations */
    int prune_num ;      /* number fo speakers to prune */

} SPRReordParam ;

typedef struct {

    PQParams pq ;             /* pre-quantization algorithm */
    int prune_num ;      /* number fo speakers to prune */

} SPRPqParams ;

typedef struct {

    int cb_size ;        /* codebook size for first iteration */
    int prune_num ;      /* number of speakers to prune */
    HMS ms ;             /* smaller size models */

} SPRHrchParams ;

typedef struct {

    int vector_step ;     /* vector step */    
    int short_term_win ;  /* window size for stabilization */

    int stab_thr_type ;   /* stabilization threshold type */
    int pr_thr_type ;     /* pruning threshold type */

    double variance_thr ; /* threshold for stabilization */
    double pruning_thr ;  /* pruning threshold */

} SPRCfParams ;

typedef struct {

    int step ;            /* vectors step */
    int number ;          /* pruned models */

} SPRStatParams ;

typedef struct {

    int step ;            /* vector step */
    double threshold ;    /* pruning threshold */

} SPRAdapParams ;

/*======================== Structures =============================*/

typedef void * HSPR ;

/*-----------------------------------------------------------------*/

/* stabilization thresholds for confidence-based pruning */
#define SPR_STBTHR_SIMPLE       0
#define SPR_STBTHR_SLOPE        1
#define SPR_STBTHR_VARIANCE     2
#define SPR_STBTHR_STDDEV       3
#define SPR_STBTHR_NONE         4

/* pruning thresholds for confidence-based pruning */
#define SPR_CFBTHR_MINMAX       0
#define SPR_CFBTHR_MEANSTD      1

/*-----------------------------------------------------------------*/

/* flags */
#define SPR_MODEL_NOTPRUNED     0
#define SPR_MODEL_PRUNED        1

/*======================== Functions ==============================*/

/* creation/deletion */
HSPR sprCreate() ;
void sprDestroy(HSPR spr) ;

/* non real-time algorithms */
int sprReordPruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRReordParam * params) ;
int sprPqBasedPruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRPqParams * params) ;
int sprHierchPruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRHrchParams * params) ;
int sprConfBasedPruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRCfParams * params) ;

int sprStaticPruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRStatParams * params) ;
int sprAdaptivePruning(HVQ vq,HMS ms,HFSET test_set,double * ret_score,SPRAdapParams * params) ;

/*======================== Functions ==============================*/

#endif /* SPRUNING_H */

