/* windows mechanism for memory leaks checking */
#if defined(_DEBUG) && defined(WIN32) && defined(_MSC_VER)
 #define _CRTDBG_MAP_ALLOC
 #include <stdlib.h>
 #include <crtdbg.h>
#else
 #include <stdlib.h>
#endif
#include <stdio.h>


#include <errorapi.h>
#include <winfunc.h>
#include <mfcc.h>


#define SIGNAL_FRAME_LENGTH 256
#define FB_NUMBER_OF_FILTERS 20
#define MFCC_ORDER 12


int main(int argc, char* argv[])
{
    FILE *infile;
	int i, count;

	signed short int audio_buffer[256];
	/*INT32* feature_buffer;*/
    INT32 ret_vector[MFCC_ORDER];
    INT32* result;
	
	HMFCC mfcc;
	MFCCParams mfcc_params;
    PreProc* ppSignal;

    /* arrange input file pointer */
    if (argc > 1)
    {
        infile = fopen(argv[1], "rb");
        if (NULL == infile)
        {
		    fprintf(stderr, "main: fopen('%s') failed\n", argv[1]);
		    return 1;
        }
    }
    else
    {
        infile = stdin;
    }

	i = frac_log2_init(8);
	if (i != 0)
	{
		fprintf(stderr, "main: frac_log2_init failed\n");
		return 1;
	}

    /* prepare audio signal pre-processor for MFCC processor */
    ppSignal = ppCreate();
    if (NULL == ppSignal)
    {
	    fprintf(stderr, "main: ppCreate() failed\n");
        if (argc > 1)
        {
            fclose(infile);
        }
	    return 1;
    }

    if (ppAddFunction(ppSignal, dspHighEmphFilter, NULL)
          == ESR_FAILED)
    {
	    fprintf(stderr, "main: ppAddFunction failed\n");
        ppDestroy(ppSignal);
        if (argc > 1)
        {
            fclose(infile);
        }
	    return 1;
    }

    /* prepare MFCC processor */
	mfcc_params.order = MFCC_ORDER;
	mfcc_params.num_mel_filters = FB_NUMBER_OF_FILTERS;
	mfcc_params.win_size = SIGNAL_FRAME_LENGTH;
	mfcc_params.spec_type = 0;
	mfcc_params.sampling_freq = 8000;
	mfcc_params.window_func = wfBlackman;
	mfcc_params.fbank = NULL;
	mfcc_params.data_preproc = ppSignal;
	mfcc_params.spectrum_preproc = NULL;
	mfcc_params.mel_spectrum_preproc = NULL;
	mfcc_params.cepstrum_preproc = NULL;
	mfcc_params.spectrum_preproc32 = NULL;
	mfcc_params.mel_spectrum_preproc32 = NULL;
	mfcc_params.cepstrum_preproc32 = NULL;
	mfcc_params.coeff_map = NULL;
	mfcc_params.flags = MFCC_FILTER_DATA | MFCC_INCLUDE_ZERO_COEFF;
	mfcc_params.shared_buffer = NULL;

	mfcc = mfccCreate(&mfcc_params);
	if (NULL == mfcc)
	{
		fprintf(stderr, "main: mfccCreate failed\n");
        ppDestroy(ppSignal);
        if (argc > 1)
        {
            fclose(infile);
        }
		return 1;
	}

    /* process the audio input */
	for (count = 0; feof(infile) == 0; count++)
	{
		if (fread(audio_buffer, 2, SIGNAL_FRAME_LENGTH, infile)
              < SIGNAL_FRAME_LENGTH)
		{
			break;
		}

		/*
		feature_buffer = mfccExtractVector(
                           mfcc,
                           audio_buffer,
                           SIGNAL_FRAME_LENGTH);
		if (NULL == feature_buffer)
		{
			fprintf(stderr, "main: mfccExtractVector failed\n");
        	mfccDestroy(mfcc);
            ppDestroy(ppSignal);
            if (argc > 1)
            {
                fclose(infile);
            }
			return 1;
		}
		*/
		result = mfccExtractVector(
                   mfcc,
                   audio_buffer,
                   SIGNAL_FRAME_LENGTH,
                   ret_vector);
		if (NULL == result)
		{
			fprintf(stderr, "main: mfccExtractVector failed\n");
        	mfccDestroy(mfcc);
            ppDestroy(ppSignal);
            if (argc > 1)
            {
                fclose(infile);
            }
			return 1;
		}

		for (i = 0; i < MFCC_ORDER; i++)
		{
            /*
			fprintf(stdout, " %ld", feature_buffer[i]);
            */
			fprintf(stdout, " %ld", ret_vector[i]);
		}
		fprintf(stdout, "\n");

		/*
		free(feature_buffer);
		*/
	}

    /* clean up */
	mfccDestroy(mfcc);
    ppDestroy(ppSignal);
    if (argc > 1)
    {
        fclose(infile);
    }

/* windows mechanism for memory leaks checking */
#if defined(_DEBUG) && defined(WIN32) && defined(_MSC_VER)
    if (_CrtDumpMemoryLeaks())
    {
        fprintf(stderr, "\nA MEMORY LEAK HAS BEEN DETECTED !!!\n\n") ;
    }
#endif

	return 0;
}

